package de.historischekommission_muenchen.oxydition.common.dialogs.elements;

import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.text.AbstractDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.DocumentFilter;

/**
 * Spezialisierter Documentfilter, der bestimmte die Texteingabe überwacht und
 * bestimmte Zeichenfolgen durch andere Zeichen ersetzt, z.B. bei
 * Anführungszeichen, EM-Striche oder hochgestellte Zahlen ersetzt.
 * 
 * @author Maximilian Schrott
 *
 */
public class OxyditionInputFilter extends DocumentFilter {

	FilterBypass currentFb;
	int currentOffset;
	int currentLength;
	String currentText;
	AttributeSet currentAttrs;
	Document currentDoc;
	int currentDocLength;
	String currentDocText;

	@Override
	public void replace(FilterBypass fb, int offset, int length, String text, AttributeSet attrs)
			throws BadLocationException {

		currentFb = fb;
		currentOffset = offset;
		currentLength = length;
		currentText = text;
		currentAttrs = attrs;
		currentDoc = fb.getDocument();
		currentDocLength = currentDoc.getLength();
		currentDocText = currentDoc.getText(0, currentDocLength);

		if (text.equalsIgnoreCase(" ")) {
			filterSpaceInsert();
		} else if (text.equalsIgnoreCase("\"")) {
			filterDoubleQuotationInsert();
		} else {
			super.replace(fb, offset, length, text, attrs);
		}
	}

	private void filterSpaceInsert() throws BadLocationException {

		String lastChar = getCharBeforeOffset(currentDocText, currentOffset);
		String twoLastChars = getSecondCharBeforeOffset(currentDocText, currentOffset) + lastChar;
		String threeLastChar = getThirdCharBeforeOffset(currentDocText, currentOffset) + twoLastChars;

		if (threeLastChar.equalsIgnoreCase("---")) {
			replaceInput("— ", 3);

		} else if (threeLastChar.equals("...")) {
			replaceInput("… ", 3);

		} else if (threeLastChar.equals("1/2")) {
			replaceInput("½ ", 3);

		} else if (threeLastChar.equals("1/4")) {
			replaceInput("¼ ", 3);

		} else if (threeLastChar.equals("3/4")) {
			replaceInput("¾ ", 3);

		} else if (twoLastChars.equalsIgnoreCase("--")) {
			replaceInput("– ", 2);

		} else if (twoLastChars.equalsIgnoreCase("^1")) {
			replaceInput("¹ ", 2);

		} else if (twoLastChars.equalsIgnoreCase("^4")) {
			replaceInput("⁴ ", 2);

		} else if (twoLastChars.equalsIgnoreCase("^5")) {
			replaceInput("⁵ ", 2);

		} else if (twoLastChars.equalsIgnoreCase("^6")) {
			replaceInput("⁶ ", 2);

		} else if (twoLastChars.equalsIgnoreCase("^7")) {
			replaceInput("⁷ ", 2);

		} else if (twoLastChars.equalsIgnoreCase("^8")) {
			replaceInput("⁸ ", 2);

		} else if (twoLastChars.equalsIgnoreCase("^9")) {
			replaceInput("⁹ ", 2);

		} else if (twoLastChars.equalsIgnoreCase("^0")) {
			replaceInput("⁰ ", 2);
		
		} else if (twoLastChars.equals("''")) {
			filterSingleQuotationInsert();

		} else {
			super.replace(currentFb, currentOffset, currentLength, currentText, currentAttrs);
		}

	}

	private String getCharBeforeOffset(String docText, int offset) {
		if (offset > 0) {
			return "" + docText.charAt(offset - 1);
		} else {
			return "";

		}
	}

	private String getSecondCharBeforeOffset(String docText, int offset) {
		if (offset > 1) {
			return "" + docText.charAt(offset - 2);
		} else {
			return "";
		}
	}

	private String getThirdCharBeforeOffset(String docText, int offset) {
		if (offset > 2) {
			return "" + docText.charAt(offset - 3);
		} else {
			return "";
		}
	}

	private void filterDoubleQuotationInsert() throws BadLocationException {
		boolean isOpeningQuotation = isOpeningDoubleQuotation(currentDocText);

		if (isOpeningQuotation) {
			super.replace(currentFb, currentOffset, currentLength, "„", currentAttrs);
		} else {
			super.replace(currentFb, currentOffset, currentLength, "“", currentAttrs);
		}
	}

	private boolean isOpeningDoubleQuotation(String docText) {

		char[] chars = docText.toCharArray();
		boolean allQutationsClosed = true;

		for (int i = 0; i < chars.length; i++) {
			char c = chars[i];

			if (c == '„') {
				allQutationsClosed = false;
			} else if (c == '“' && !allQutationsClosed) {
				allQutationsClosed = true;
			}
		}
		return allQutationsClosed;
	}

	private void filterSingleQuotationInsert() throws BadLocationException {
		boolean isOpeningSingleQuotation = isOpeningSingleQuotation(currentDocText);

		if (isOpeningSingleQuotation) {
			replaceInput("‚", 2);
		} else {
			replaceInput("‘", 2);
		}
	}

	private boolean isOpeningSingleQuotation(String docText) {
		char[] chars = docText.toCharArray();
		boolean allQutationsClosed = true;

		for (int i = 0; i < chars.length; i++) {
			char c = chars[i];

			if (c == '‚') {
				allQutationsClosed = false;
			} else if (c == '‘' && !allQutationsClosed) {
				allQutationsClosed = true;
			}
		}
		return allQutationsClosed;
	}

	private void replaceInput(String newInput, int removedCharacters) throws BadLocationException {
		super.replace(currentFb, currentOffset, currentLength, newInput, currentAttrs);

		if (removedCharacters == 3) {
			SwingUtilities.invokeLater(removeThreeCharactersBeforeOffset);
		} else if (removedCharacters == 2) {
			SwingUtilities.invokeLater(removeTwoCharactersBeforeOffset);
		}
	}

	Runnable removeLastTwoCharacters = new Runnable() {

		@Override
		public void run() {
			Document doc = currentFb.getDocument();
			int docLength = doc.getLength();
			try {
				doc.remove(docLength - 4, 2);
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}
	};

	Runnable removeLastThreeCharacters = new Runnable() {

		@Override
		public void run() {
			Document doc = currentFb.getDocument();
			int docLength = doc.getLength();
			try {
				doc.remove(docLength - 5, 3);
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}
	};

	Runnable removeTwoCharactersBeforeOffset = new Runnable() {

		@Override
		public void run() {
			Document doc = currentFb.getDocument();

			try {
				doc.remove(currentOffset - 2, 2);
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}
	};

	Runnable removeThreeCharactersBeforeOffset = new Runnable() {

		@Override
		public void run() {
			Document doc = currentFb.getDocument();
			try {
				doc.remove(currentOffset - 3, 3);
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}
	};

	public static void setForInput(JTextField inputField) {
		((AbstractDocument) inputField.getDocument()).setDocumentFilter(new OxyditionInputFilter());
	}

	public static void setForInput(JTextArea inputArea) {
		((AbstractDocument) inputArea.getDocument()).setDocumentFilter(new OxyditionInputFilter());
	}
}
