package de.historischekommission_muenchen.oxydition.common.operations;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.regex.Pattern;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.ReadRegisterOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.StandardReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxygen-Author-Operation, mit der die Benutzer*innen einen mit der
 * Registerdatenbank verknüpften Platzhalter in ein Dokument einfügen können.
 * Der Platzhalter markiert die Stelle, an der zu einem späteren Zeitpunkt Daten
 * des gewählten RDB-Eintrags eingefügt werden sollen, die zum Zeitpunkt der
 * Bearbeitung noch nicht (vollständig) vorliegen müssen. Der Typ des
 * Platzhalters wird im type-Attribut festgehalten und durch Operationsparameter
 * festgelegt. Ebenso können durch einen Operationsparameter Daten aus dem
 * RDB-Eintrag ausgewählt werden, die in den Platzhalter als Text eingefügt
 * werden. Dieser dient zur leichteren Identifikation während der Bearbeitung
 * und wird in der fertigen Edition ebenfalls ersetzt. Ebenso werden die
 * RDB-Pfade durch Operationsargument festgelegt.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertRegisterPlaceholder extends ReadRegisterOperation {

	protected static final String ARGUMENT_ID = "Referenzdaten";
	protected static final String ARGUMENT_TYPE = "Platzhalter-Typ";

	protected static String idArgDescription = "Die Daten die in den Platzhalter eingetragen werden sollen."
			+ "\nDie Daten können nach dem selben Muster wie bei \"Vorschlag\" ausgewählt"
			+ "\nund mit Strings in Anführungszeichen ergänzt werden werden, z.B."
			+ "\n\"//*[local-name()='persName' and @type='Lesename']+\" \"+//*[local-name()='note' and @type='Qualifikator']\"";
	protected static String typeArgDescription = "Die Art des Platzhalters. Wird im Attibute 'type' des Platzhalterelements festgehalt, z.B. \"Biogramm\"";

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {
			new ArgumentDescriptor("URL", ArgumentDescriptor.TYPE_STRING, urlArgDescription),
			new ArgumentDescriptor("Basisknoten", ArgumentDescriptor.TYPE_STRING, nodeArgDescription),
			new ArgumentDescriptor("Vorschlag", ArgumentDescriptor.TYPE_STRING, expressionArgDescription),
			new ArgumentDescriptor("Referenzdaten", ArgumentDescriptor.TYPE_STRING, idArgDescription),
			new ArgumentDescriptor("Platzhalter-Typ", ArgumentDescriptor.TYPE_STRING, typeArgDescription) };

	@Override
	public String getDescription() {

		return "Öffnet ein Dialogfenster, in dem die festgelegten Basisknoten aus der festgelegten Datei "
				+ "in der festgelegten Vorschlagsform den Benutzer*innen zur Auswahl gestellt werden. "
				+ "Wählen sie einen Eintrag aus und wird ein Platzhalter des festgelegten Typs für den entsprechenden Knoten, "
				+ "an der Position des Textcursors in das Dokument eingefügt";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		if (!validateArguments(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			String typeArgString = (String) args.getArgumentValue(ARGUMENT_TYPE);

			ReadRegisterBasic registerReader = getRegisterEntries(args);
			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();
			String titleText = typeArgString + "-Platzhalter einfügen";

			SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame, titleText, eintraege, ids);
			ListSelectionBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertPlaceholderFragment(input, typeArgString);
			}
		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			e.printStackTrace();
			OperationUtility.handleGenericException(oxygenFrame, e);
		}

	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

	private boolean validateArguments(ArgumentsMap args) {

		boolean allValid = true;

		allValid = validateBaseArguments(args);

		if (!OperationUtility.isValidArgument(args.getArgumentValue(ARGUMENT_ID))) {
			allValid = false;
		}
		if (!OperationUtility.isValidArgument(args.getArgumentValue(ARGUMENT_TYPE))) {
			allValid = false;
		}
		return allValid;
	}

	@Override
	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {
		String urlArgString = (String) args.getArgumentValue(ARGUMENT_URL);
		String nodeArgString = (String) args.getArgumentValue(ARGUMENT_NODE);
		String expressionArgString = (String) args.getArgumentValue(ARGUMENT_EXPRESSION);
		String idArgString = (String) args.getArgumentValue(ARGUMENT_ID);

		// @xml:id muss immer ausgelesen werden
		String idValue = "@xml:id+\"$#\"+" + idArgString;

		return new StandardReadRegister(urlArgString, nodeArgString, expressionArgString, idValue);
	}

	private void insertPlaceholderFragment(ListSelectionBean input, String typeArgString)
			throws AuthorOperationException {
		String placeholderFragment = constructPlaceholderFragment(input, typeArgString);
		documentController.insertXMLFragment(placeholderFragment, caretPosition);
	}

	private String constructPlaceholderFragment(ListSelectionBean input, String typeArgString) {

		String idValue = input.id;
		String[] idValueSplit = idValue.split(Pattern.quote("$#"));

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<ref " + XMLUtility.TEIXMLNS + " type='" + typeArgString + "_Platzhalter' target='"
				+ idValueSplit[0] + "'>");
		fragmentBuilder.append(typeArgString + ":");

		// Falls die Referenzdaten nur aus der ID bestehen, wird diese in den
		// Platzhalter eingefügt, sonst die überigen Referenzdaten
		if (idValueSplit.length == 1) {
			fragmentBuilder.append(" " + idValueSplit[0]);
		} else {

			for (int i = 1; i < idValueSplit.length; i++) {
				String placeholderText = XMLUtility.escapeXML(idValueSplit[i]);

				fragmentBuilder.append(" " + placeholderText);
			}
		}

		fragmentBuilder.append("</ref>");

		return fragmentBuilder.toString();
	}
}
