package de.historischekommission_muenchen.oxydition.common.operations;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.ReadRegisterOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxygen-Author-Operation, mit der die Benutzer*innen einen markierten
 * Textauschnitt mit einem <rs>-Element umgeben lassen können, das auf einen
 * Eintrag aus der Registerdatenbank verweist. Diesen Eintrag wählen die
 * Benutzer*innen zuvor in einem Dialogfenster aus einer Vorschlagsliste aus.
 * Die Pfade zur Registerdatenbank und den Einträge werden in den
 * Operationsargumenten festgelegt.
 * 
 * @author Maximilian Schrott
 *
 */
public class TagSelectionOperation extends ReadRegisterOperation {

	private static String ARGUMENT_TYPE = "Typ der Referenz";
	private static String typeArgDescription = "Der Typ der Referenz. "
			+ "Wird im Attribut 'type' des eingefügten <rs>-Elements festgehalten. Z.B.: Ort";

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {
			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription),
			new ArgumentDescriptor(ARGUMENT_NODE, ArgumentDescriptor.TYPE_STRING, nodeArgDescription),
			new ArgumentDescriptor(ARGUMENT_EXPRESSION, ArgumentDescriptor.TYPE_STRING, expressionArgDescription),
			new ArgumentDescriptor(ARGUMENT_TYPE, ArgumentDescriptor.TYPE_STRING, typeArgDescription) };

	@Override
	public String getDescription() {

		return "Öffnet ein Dialogfenster, in dem die festgelegten Basisknoten aus der festgelegten Datei "
				+ "in der festgelegten Vorschlagsform den Benutzer*innen zur Auswahl gestellt werden. "
				+ "Wählen sie einen Eintrag aus und wird eine Referenz auf den entsprechenden Knoten, "
				+ "um die Textauswahl herum in Form eine <rs>-Elements eingefügt";
	}

	@Override
	public void doOperation(final AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		String selectedText = editorAccess.getSelectedText().trim();

		if (selectedText.isEmpty()) {
			displayNoSelectionMessage();
			return;
		}

		if (!validateArguments(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			String typeArgVal = (String) args.getArgumentValue(ARGUMENT_TYPE);
			ReadRegisterBasic registerReader = getRegisterEntries(args);

			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();

			String frameName = "Auswahl als " + typeArgVal + " markieren";

			SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame, frameName, eintraege, ids);
			dlg.updateSearchRegisterField(selectedText);
			ListSelectionBean input = dlg.displayDialog();

			if (input.isFilled) {
				tagSelection(input, typeArgVal);
			}

		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			OperationUtility.handleGenericException(oxygenFrame, e);
		}
	}

	protected boolean validateArguments(ArgumentsMap args) {
		boolean allValid = true;

		allValid = validateBaseArguments(args);

		if (!OperationUtility.isValidArgument(ARGUMENT_TYPE)) {
			allValid = false;
		}
		return allValid;
	}

	protected void tagSelection(ListSelectionBean input, String typeArgVal) throws AuthorOperationException {
		int selectionStart = editorAccess.getSelectionStart();
		int selectionEnd = editorAccess.getSelectionEnd();

		String tagFragment = constructTagFragment(input, typeArgVal);
		documentController.surroundInFragment(tagFragment, selectionStart, selectionEnd - 1);
	}

	protected String constructTagFragment(ListSelectionBean input, String typeArgVal) {

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<rs " + XMLUtility.TEIXMLNS + " type='" + typeArgVal + "' ");
		fragmentBuilder.append("key='" + input.id + "'/>");

		return fragmentBuilder.toString();
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
