/**
 * Stellt die Methode findNextTermOffset() nach außen zur Verfügung mit 
 * der (mit Hilfe eines von der aufrufenden Klasse übergebenen TextContentIterators)
 * das nächste Vorkommen eines gesuchten Textausschnitts
 * innerhalb eines Dokuments gefunden werden kann
 *
 *@author Maximilian Schrott
 *
 */
package de.historischekommission_muenchen.oxydition.common.utilities;

import ro.sync.ecss.extensions.api.content.TextContentIterator;
import ro.sync.ecss.extensions.api.content.TextContext;

public class OxygenDocumentTextSearcher {

	private TextContentIterator iterator; // Der Iterator für das XML-Dokumnet,
											// der von der steuernden Klasse
											// übergeben wurde
	private TextContext context; // Der XML-Context auf dem der Iterator aktuell
									// steht
	private String checkText; // Der textinhalt des akutellen Contextes
	private String searchTerm; // Gesuchter Textausschnitt
	private int searchTermLength; // Länge des gesuchten Textausschnitts
	private int lastIndex;

	/**
	 * Konstruktor
	 * 
	 * @param iterator   - TextIterator der von der Klasse, die die Suche verwaltet
	 *                   und verarbeitet weitergegeben wird
	 * @param searchTerm - Der Textausschnitt nachdem gesucht werden soll
	 */
	public OxygenDocumentTextSearcher(TextContentIterator iterator, String searchTerm) {

		this.iterator = iterator;
		this.searchTerm = searchTerm;

		searchTermLength = searchTerm.length();

		lastIndex = -1; // Index des letzten Treffers im aktuellen Context; -1
						// bedeutet, dass im aktuellen Context noch kein Treffer
						// gefunden wurde
		checkText = ""; // T
	}

	/**
	 * Durchsucht mithilfe des Iterators der Klasse das gesamte Dokument nach dem
	 * nächsten Vorkommen des gesuchten Wortes (sofern noch einer vorhanden ist)
	 * 
	 * @return das Offsets des gefundenen Treffers im Dokument; Falls keine weiteren
	 *         Treffer vorhanden wird -1 als Signalwert zurückgegeben
	 */
	public int findNextTermOffset() {

		// Wenn bereits ein Treffer im aktuellen Context gefunden wurde, diesen
		// Context weiter durchsuchen
		if (lastIndex >= 0) {

			lastIndex = findTermInContext(lastIndex + searchTermLength);

			// Wurde ein weiterer Treffer im aktuellen Context gefunden, dessen
			// Offset im Dokument (Startoffset des Contextes + Trefferindex im
			// Context) zurückgeben, sonst die weiteren Context
			// durchsuchen
			if (lastIndex >= 0) {
				return lastIndex + context.getTextStartOffset();
			} else {
				lastIndex = findNextContextWithTerm();
			}
		}
		// Gibt es noch keinen aktuellen Context mit Treffer, die weiteren
		// Contexte durchsuchen
		else {
			lastIndex = findNextContextWithTerm();
		}

		// Wurde ein Treffer in einem weiteren Context gefunden, dessen Offset
		// im Dokument zurückgeben; Ansonsten mit der Rückgabe -1 anzeigen, dass
		// es keine weiteren Treffer im Dokument gibt
		if (lastIndex >= 0) {
			return context.getTextStartOffset() + lastIndex;
		} else {
			return -1;
		}

	}

	/**
	 * Sucht innerhalb des akutellen Contextes nach dem gesuchten Textausschnitt
	 * 
	 * @param searchStartIndex - Anfangspunkt der Suche (>0 falls im aktuellen
	 *                         Context bereits ein Treffer gefunden wurde)
	 * @return den Startindex des gesuchten Textausschnitts innerhalb des Textes,
	 *         falls vorhanden; Wenn nicht -1
	 */
	private int findTermInContext(int searchStartIndex) {

		return checkText.indexOf(searchTerm, searchStartIndex);

	}

	/**
	 * Benutzt den Iterator der Klasse um den nächsten XML-Context zu finden, in dem
	 * der gesuchte Textausschnitt enthalten ist
	 * 
	 * @return der Index des gesuchten Textausschnitts innerhalb den Contextes,
	 *         falls vorhanden; Ansonsten -1 als Zeichen dafür, dass es keine
	 *         weiterern Treffer für den gesuchten Textauschnitt gibt
	 */
	private int findNextContextWithTerm() {

		// Rückgabewert standartmäßig auf den Wert für keine weiteren Treffer
		// setzen
		int returnIndex = -1;

		// Schleife durchläuft mit dem Iterator alle Contexte des Dokuments und
		// prüft bei jedem ob der gesuchte Textausschnitt enthalten ist.
		while (iterator.hasNext()) {

			context = iterator.next();
			checkText = (String) context.getText();
			returnIndex = findTermInContext(0);

			// Wenn der Context den gesuchten Textausschnitt enthielt, wird die
			// Schleife abgebrochen und der Index des Treffers innerhalb des
			// Contexts zurückgegeben
			if (returnIndex >= 0) {

				return returnIndex;
			}

		}

		// Wurde die Schleife abgeschlossen ohne einen weiteren Context mit
		// Treffer zu finden wird -1 als Signalwert zurückgegeben
		return returnIndex;
	}

	/**
	 * Get-Methode für den aktuell durchsuchten Context
	 * 
	 * @return der Context auf dem der Iterator aktuell steht
	 */
	public TextContext getContext() {
		return context;
	}

	/**
	 * Liefert den Offset des aktuellen Suchtreffers
	 * 
	 * @return Offset im Dokument an dem der aktuelle Suchtreffer beginnt
	 */
	public int getLastFoundOffset() {
		return context.getTextStartOffset() + lastIndex;
	}
}
