package de.historischekommission_muenchen.oxydition.protedition.common.operations;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.RomanNumber;
import de.historischekommission_muenchen.oxydition.protedition.common.beans.InsertTopicReferenceBean;
import de.historischekommission_muenchen.oxydition.protedition.common.dialogs.InsertTopicReferenceDialog;
import de.historischekommission_muenchen.oxydition.protedition.common.utilities.ProtocolUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * 
 * Abstrakte Oberklasse für Operationen mit denen die Benutzer*innen Verweise
 * auf Tagesordnungspunkt innerhalb der Edition einfügen können (als
 * <ref>-Element). Dabei wird aus den Eingaben der Benutzer*innen die TOP-ID
 * rekonsturiert, statt dass sie diese heraussuchen und eintragen müssen. Erbende
 * Klassen müssen nur noch einstellen auf welche (editionsspezifische) Bandliste
 * sie zugreifen wollen.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class InsertTopicReference extends OxyditionOperation {

	private int protocolVolumeIndex;
	protected String[] volumeTitles;
	protected String[] volumeIds;

	@Override
	public String getDescription() {
		return "Fügt eine Referenz auf irgendeinen Tagesordungspunkt aus der Edition an der Stelle des Textcursors ein";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		getVolumeInformation();

		String protocolVolumeID = ProtocolUtility.getVolumeID(documentController);

		// Wurde kein hinterleges Bandkürzel im Dokument gefunden (z.B. in einer
		// RDB-Datei), den aktuellsten Band vorauswählen
		if (protocolVolumeID == null) {
			protocolVolumeIndex = volumeIds.length - 1;
		} else {
			protocolVolumeIndex = volumeIDtToVolumeIndex(protocolVolumeID);
		}
		InsertTopicReferenceDialog dlg = new InsertTopicReferenceDialog(oxygenFrame,
				"Verknüpfung auf Tagesordnungspunkt einfügen", true, volumeTitles);

		InsertTopicReferenceBean inputBean = dlg.displayDialog(protocolVolumeIndex);

		if (inputBean.isFilled) {
			String topicReferenceFragment = constructTopicReferenceXMLFragment(inputBean);
			documentController.insertXMLFragment(topicReferenceFragment, caretPosition);
		}
	}

	protected abstract void getVolumeInformation();

	@Override
	public ArgumentDescriptor[] getArguments() {
		return null;
	}

	// Findet Arrayindex für ein Bandkürzel aus der Bandkürzelliste heraus
	private int volumeIDtToVolumeIndex(String protocolVolumeID) {

		for (int i = 0; i < volumeIds.length; i++) {

			if (protocolVolumeID.equalsIgnoreCase(volumeIds[i])) {
				return i;
			}
		}

		return -1;
	}

	// Stellt das XML-Fragment für den Tagesordnungspunktverweis
	private String constructTopicReferenceXMLFragment(InsertTopicReferenceBean inputBean) {

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<rs xmlns='http://www.tei-c.org/ns/1.0' type='Top_Referenz' key='");
		fragmentBuilder.append(buildReferenceKey(inputBean) + "'>");
		fragmentBuilder.append(createReferenceText(inputBean) + "</rs>");

		return fragmentBuilder.toString();
	}

	// Stellt den Text für das key-Attribut zusammen
	private String buildReferenceKey(InsertTopicReferenceBean inputBean) {

		StringBuilder keyBuilder = new StringBuilder();
		keyBuilder.append(createVolumeKeyFragment(inputBean.volumeIndex));
		keyBuilder.append("p" + prefixZerosForLength(inputBean.protocolNumber, 3));
		keyBuilder.append("t" + prefixZerosForLength(inputBean.topicNumber, 2));
		keyBuilder.append(createSubTopicKeyFragment(inputBean.subTopicNumber));
		keyBuilder.append(createSubTopicKeyFragment(inputBean.subSubTopicNumber));
		keyBuilder.append(createSubTopicKeyFragment(inputBean.subSubSubTopicNumber));

		return keyBuilder.toString();
	}

	// Stellt den Text für den Textinhalt des <rs>-Elements zusammen
	private String createReferenceText(InsertTopicReferenceBean inputBean) {

		StringBuilder textBuilder = new StringBuilder();

		if (protocolVolumeIndex != inputBean.volumeIndex) {
			textBuilder.append(createVolumeText(inputBean.volumeIndex));
		}

		textBuilder.append("Nr.\u00A0" + inputBean.protocolNumber);
		textBuilder.append(createTopicText(inputBean.topicNumber));
		textBuilder.append(createSubTopicsText(inputBean));

		return textBuilder.toString();
	}

	private String createVolumeKeyFragment(int volumeIndex) {

		return volumeIds[volumeIndex];

	}

	// Stellt ein Unterpunktssuffix für das key-Attribut zusammen (Buchstaben
	// werden großgestellt zur Vereinheitlichung)
	private String createSubTopicKeyFragment(String subTopicNumber) {

		subTopicNumber = subTopicNumber.toUpperCase();

		if (subTopicNumber.isEmpty()) {
			return "";
		} else {

			// Einstellige TopNummern ergänzen mit führender Null
			if (subTopicNumber.matches("[0-9]")) {
				subTopicNumber = prefixZerosForLength(subTopicNumber, 2);
			}

			return "u" + subTopicNumber;
		}

	}

	// Stellt den Tagesordnungsnummern-Teil des <rs>-Textinhalts zusammen
	private String createTopicText(String topicNumber) {

		int topicNumberDecimal = Integer.parseInt(topicNumber);
		String topicNumberRoman = RomanNumber.toRoman(topicNumberDecimal);

		return " TOP\u00A0" + topicNumberRoman;

	}

	private Object createVolumeText(int volumeIndex) {

		String volumeTitle = volumeTitles[volumeIndex];

		String[] volumeTitleSplit = volumeTitle.split(" ", 3);

		StringBuilder volumeTextBuilder = new StringBuilder();
		volumeTextBuilder.append("<hi>" + volumeTitleSplit[0]);
		volumeTextBuilder.append(" " + volumeTitleSplit[1] + "</hi> ");
		if (volumeTitleSplit.length > 2) {
			volumeTextBuilder.append(volumeTitleSplit[2] + " ");
		}
		return volumeTextBuilder.toString();

	}

	// Stellt das Unterpunktsuffix für den <rs>-Textinhalt zusammen
	private String createSubTopicsText(InsertTopicReferenceBean inputBean) {

		String subTopicsText = "";
		if (!inputBean.subTopicNumber.isEmpty()) {

			subTopicsText += "/" + inputBean.subTopicNumber;
			subTopicsText += inputBean.subSubTopicNumber;
			subTopicsText += inputBean.subSubSubTopicNumber;
		}

		return subTopicsText;
	}

	// fügt zu einer Zahl als String führende Nullen hinzu, bis eine bestimmte
	// Textlänge erreicht ist
	private String prefixZerosForLength(String number, int targetLength) {

		while (number.length() < targetLength) {
			number = "0" + number;
		}

		return number;
	}
}
