package de.historischekommission_muenchen.oxydition.protedition.common.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen Tagesordnungspunkt
 * in der Tagesordnung mit demselben Tagesordnungspunkt im Protokolltext
 * verknüpfen können. Hierfür müssen die Benutzer*innen den TOP in der
 * Tagesordnung markieren und die Operation ausführen. Diese versucht daraufhin
 * automatisch den passenden TOP im Protokoll zu finden und die Verküpfen (mit
 * einem <rs>-Element) herzustellen. Gelingt diese nicht öffnet sie stattdessen
 * ein Dialogfenster, in dem die Benutzer*innen den gesuchten TOP manuell
 * auswählen können.
 * 
 * @author Maximilian Schrott
 * 
 */
public class LinkTopicInternal extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Erlaubt es den Benutzer*innen eine markierte Textstelle mit einem Tagesordnungspunkt innerhalb desselben Protokolls zu verbinden.\n"
				+ "Die Benutzer*innen markieren hierfür den Titel eine TOPs in der Tagesordnung und führt die Operation aus. "
				+ "Die Operation dursucht darauf die TOP-Überschriften innerhalb des Protokolls nach einer Übereinstimmung.\n"
				+ "Findet sie eine, wird dieses automatiosch mit der markierten Textstelle verknüpft."
				+ "Andernfalls öffnet sich ein Dialogfenster, in dem die Benutzer*innen den gesuchten TOP selbst aussuchen können.";
	}

	@Override
	public void doOperation(final AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		String selectionText = authorAccess.getEditorAccess().getSelectedText();
		int selectionStartOffset = authorAccess.getEditorAccess().getSelectionStart();
		int selectionEndOffset = authorAccess.getEditorAccess().getBalancedSelectionEnd();

		if (selectionText.isEmpty()) {
			displayNoSelectionMessage();
			return;
		}

		try {

			AuthorNode[] topicHeadList = getTopicHeadList(documentController);

			String[] eintraege = constructTopicTitleList(topicHeadList);
			String[] ids = constructTopicIdList(topicHeadList);

			String cleanedSelectionText = cleanUpSelectionText(selectionText);

			int titleMatchIndex = findSelectionTitleMatch(cleanedSelectionText, eintraege);

			// Genau eine Überstimmung ziwschen markiertem Textausschnitt und einer
			// TOP-Überschrift gefunden? Dann automatisch referenzieren, sonst Dialogfenster
			// öffnen und Benutzer*innen auswählen lassen
			if (titleMatchIndex >= 0) {
				String topicId = ids[titleMatchIndex];

				insertReference(authorAccess, topicId, selectionStartOffset, selectionEndOffset);
			} else {
				SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame,
						"Tagesordnungspunkt verknüpfen", eintraege, ids);
				ListSelectionBean input = dlg.displayDialog();

				if (input.isFilled) {
					insertReference(authorAccess, input.id, selectionStartOffset, selectionEndOffset);
				}
			}
		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}

	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

	/**
	 * Durchläuft alle <head>-Elemente in TOPs und fügt in die Rückgabeliste
	 * abwechselnd die Nummer des TOPs und den Titel ein (ohne Fußnotentexte);
	 * Wandelt <add>-Elemente in eckige Klammern um
	 */
	private AuthorNode[] getTopicHeadList(AuthorDocumentController documentcontroller) throws AuthorOperationException {

		String topicTextXPath = "//div[@type='Top']/head//text()[if ( normalize-space(.) != '' and not(ancestor::note) and not(ancestor::rs)) then if( ancestor::add) then concat('[', . , ']') else . else ('')]";

		return documentcontroller.findNodesByXPath(topicTextXPath, false, false, true);
	}

	private String[] constructTopicTitleList(AuthorNode[] topicHeads) throws BadLocationException {

		String[] topicTitles = new String[topicHeads.length / 2];
		int j = 0;

		for (int i = 0; i < topicHeads.length; i += 2) {

			AuthorNode numNode = topicHeads[i];
			AuthorNode textNode = topicHeads[i + 1];

			String topicTitle = numNode.getTextContent() + " " + textNode.getTextContent();

			topicTitles[j] = topicTitle;
			j++;
		}

		return topicTitles;
	}

	private String[] constructTopicIdList(AuthorNode[] topicHeads) {

		String[] topicIds = new String[topicHeads.length / 2];
		int j = 0;

		for (int i = 0; i < topicHeads.length; i += 2) {

			AuthorElement divNode = findTopicDivNode(topicHeads[i]);

			if (XMLUtility.elementTypeEquals(divNode, "Top")) {

				topicIds[j] = XMLUtility.getAttributeRawValue(divNode, "xml:id");
				j++;
			}
		}

		return topicIds;
	}

	private String cleanUpSelectionText(String selectionText) {

		String cleanedText = selectionText.trim();
		cleanedText = cleanedText.replace("[", "");
		cleanedText = cleanedText.replace("]", "");

		char lastChar = cleanedText.charAt(cleanedText.length() - 1);

		if (lastChar == '.') {
			cleanedText = cleanedText.substring(0, cleanedText.length() - 1);
		}

		return cleanedText;
	}

	/**
	 * Gibt die Position im Array zurück, falls genau ein Match zwischen TOP-Titeln
	 * und markiertem Textausschnitt gefunden wurde; Andernfalls wird "-1" als
	 * Fehlersignal zurückgegeben
	 */
	private int findSelectionTitleMatch(String selectionText, String[] eintraege) {

		int numMatches = 0;
		int foundIndex = -1;

		for (int i = 0; i < eintraege.length; i++) {

			if (selectionText.equals(eintraege[i])) {
				numMatches++;
				foundIndex = i;
			}
		}

		if (numMatches > 1) {
			return -1;
		} else {
			return foundIndex;
		}
	}

	private AuthorElement findTopicDivNode(AuthorNode baseNode) {

		AuthorNode topicDivNode = baseNode.getParent();

		while (topicDivNode.getDisplayName() != "div" && topicDivNode.getDisplayName() != "TEI") {
			topicDivNode = topicDivNode.getParent();
		}

		return (AuthorElement) topicDivNode;
	}

	private void insertReference(AuthorAccess authorAccess, String id, int startOffset, int endoffset)
			throws AuthorOperationException {

		String fragment = "<rs xmlns='http://www.tei-c.org/ns/1.0' type='Top_Link' key='#" + id + "'/>";

		authorAccess.getDocumentController().surroundInFragment(fragment, startOffset, endoffset - 1);

	}
}
