package de.historischekommission_muenchen.oxydition.register.operations;

import java.io.IOException;

import javax.swing.text.BadLocationException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.ReadRegisterOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.StandardReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einem RDB-Eintrag einen
 * Querverweis auf einen ausgewählten, anderen Eintrag aus der selben RDB-Datei
 * hinzufügen können. Hierfür wird ein <note @type="Querverweis>-Element zum
 * Ausgangseintrag hinzugfügt mit der xml:id des referenzierten Eintrags als
 * target-Attribut.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterAddCrossReference extends ReadRegisterOperation {

	private static String ARGUMENT_ENTRY = "Registereintragsknoten";

	private static String entryArgDescription = "Der lokale Name der Knoten, innerhalb derer ein Querverweis erlaubt ist, z.B. \"person\"";

	protected static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {

			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription),
			new ArgumentDescriptor(ARGUMENT_NODE, ArgumentDescriptor.TYPE_STRING, nodeArgDescription),
			new ArgumentDescriptor(ARGUMENT_EXPRESSION, ArgumentDescriptor.TYPE_STRING, expressionArgDescription),
			new ArgumentDescriptor(ARGUMENT_ENTRY, ArgumentDescriptor.TYPE_STRING, entryArgDescription), };

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, dass die durch die Operationsargumente festgelegten Einträge aus der voreingestellten RDB-Datei anzeigt. "
				+ "die Benutzer*innen können einen dieser Einträge auswählen. "
				+ "Dem Registereintrag an der Position des Textcursors wird daraufhin ein Querverweis auf den gewählten Eintrag hinzugefügt.";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		if (!validateArguments(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {

			String entryNodeName = (String) args.getArgumentValue(ARGUMENT_ENTRY);

			AuthorElement sourroundElement = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			AuthorElement entryNode = findEntryNode(sourroundElement, entryNodeName);

			if (entryNode == null) {
				displayInvalidCaretPositionMessage();
				return;
			}

			ReadRegisterBasic registerEntries = getRegisterEntries(args);

			String[] eintraege = registerEntries.getEintrag();
			String[] ids = registerEntries.getID();

			SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame, "Querverweis hinzuverfügen",
					eintraege, ids);
			ListSelectionBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertCrossReference(input, entryNode, documentController);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			e.printStackTrace();
			OperationUtility.handleGenericException(oxygenFrame, e);
		}

	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

	private boolean validateArguments(ArgumentsMap args) {

		boolean allValid = true;

		allValid = validateBaseArguments(args);

		if (!OperationUtility.isValidArgument(args.getArgumentValue(ARGUMENT_ENTRY))) {
			allValid = false;
		}

		return allValid;
	}

	private AuthorElement findEntryNode(AuthorElement surroundNode, String entryNodeName) {

		AuthorElement checkNode = surroundNode;
		while (!XMLUtility.elementDisplayNameEquals(checkNode, "TEI")) {

			if (XMLUtility.elementDisplayNameEquals(checkNode, entryNodeName)) {
				return checkNode;
			} else {
				checkNode = XMLUtility.getNodeParentElement(checkNode);
			}
		}

		return null;
	}

	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {

		String urlArgString = (String) args.getArgumentValue(ARGUMENT_URL);
		String nodeArgString = (String) args.getArgumentValue(ARGUMENT_NODE);
		String expressionArgString = (String) args.getArgumentValue(ARGUMENT_EXPRESSION);

		return new StandardReadRegister(urlArgString, nodeArgString, expressionArgString, idVal);
	}

	private void insertCrossReference(ListSelectionBean input, AuthorElement entryNode,
			AuthorDocumentController documentController) throws AuthorOperationException {

		String crossReferenceFragment = constructCrossReferenceFragment(input);
		int entryNodeEnd = entryNode.getEndOffset();
		documentController.insertXMLFragment(crossReferenceFragment, entryNodeEnd);
	}

	protected String constructCrossReferenceFragment(ListSelectionBean input) {

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<note " + XMLUtility.TEIXMLNS + " type='Querverweis' ");
		fragmentBuilder.append("target='" + input.id + "'/>");

		return fragmentBuilder.toString();

	}

	protected String getInvalidCaretPositionMessage() {
		return "An dieser Textcursorposition darf kein Querverweis eingefügt werden.";
	}

}
