package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.JFrame;
import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.beans.LiteratureTitlePartBean;
import de.historischekommission_muenchen.oxydition.register.beans.NewLiteraturTitleBean;
import de.historischekommission_muenchen.oxydition.register.dialogs.RegisterLiteratureTitleDialog;
import de.historischekommission_muenchen.oxydition.register.utilities.RegisterUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Operation mit der die Benutzer*innen einen neuen Literatur-Eintrag in die RDB
 * einfügen können.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterNewLiteratureTitle extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen einen neuen Literatur für das Register anlegen und einfügen können."
				+ "Der neue Titel wird an der Cursorposition in das aktuelle Dokument eingefügt";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		AuthorElement surroundNode;
		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			if (!nodeWithinLiteraturList(surroundNode)) {
				displayInvalidCaretPositionMessage();
				return;
			}

			NewLiteraturTitleBean input = displayNewLiteraturTitleDialog(oxygenFrame);

			if (input.isFilled) {
				insertLiteratureTitleFragment(input);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}

	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

	// Überprüft ob der Elternknoten des übergebenen Knoten eine
	// <listBibl>-Element mit @type="Literatur" ist
	private boolean nodeWithinLiteraturList(AuthorElement checkNode) {

		if (XMLUtility.elementDisplayNameEquals(checkNode, "listBibl")
				&& XMLUtility.elementTypeEquals(checkNode, "Literatur")) {
			return true;
		}

		return false;

	}

	// Initalisiert das Dialogfenster, führt es aus und gibt die
	// Benutzereingaben zurück
	private NewLiteraturTitleBean displayNewLiteraturTitleDialog(JFrame oxygenFrame) {

		NewLiteraturTitleBean input = new NewLiteraturTitleBean();

		RegisterLiteratureTitleDialog dlg;
		dlg = new RegisterLiteratureTitleDialog(oxygenFrame, "Neuen Literaturtitel ins Register einfügen");
		input = dlg.displayDialog();

		return input;

	}

	private void insertLiteratureTitleFragment(NewLiteraturTitleBean input) throws AuthorOperationException {

		String newLiteraturTitle = constructNewLiteratureTitleXMLFragment(input);
		documentController.insertXMLFragment(newLiteraturTitle, caretPosition);

	}

	// Konstruiert aus den übergeben Benutzereingaben das XML-Fragment für den
	// neuen Registereintrag
	private String constructNewLiteratureTitleXMLFragment(NewLiteraturTitleBean input) {

		StringBuilder fragmentBuilder = new StringBuilder();
		String xmlid = createLiteratureTitleId(input.fullTitle, input.date);

		fragmentBuilder.append("<bibl xmlns='http://www.tei-c.org/ns/1.0' xml:id='" + xmlid + "'>");
		fragmentBuilder.append("<title type='Volltitel'>" + constructFullTitleFragment(input.fullTitle) + "</title>");
		fragmentBuilder.append("<title type='Kurztitel'>" + constructShortTitleFragment(input.shortTitle) + "</title>");
		fragmentBuilder.append("<date>" + input.date + "</date>");
		fragmentBuilder.append("</bibl>");

		String fragment = fragmentBuilder.toString();

		return fragment;
	}

	// Trifft die Unterscheidung zwischen Titel mit|ohne hervorgehobenem
	// Abschnitt
	private String createLiteratureTitleId(LiteratureTitlePartBean[] fullTitle, String date) {

		for (int i = 0; i < fullTitle.length; i++) {

			LiteratureTitlePartBean fullTitlePart = fullTitle[i];

			if (fullTitlePart.isHighlighted) {
				return constructLiteratureID(fullTitlePart.titlePart, date);
			}
		}

		return constructLiteratureID(fullTitle[0].titlePart, date);
	}

	// Konstruiert die Literatur ID so, dass sie maximal 20 Zeichen lang ist und
	// mit dem Veröffentlichungsdatum endet
	private String constructLiteratureID(String titleHiglight, String date) {

		String clippedTitle = titleHiglight;
		if (titleHiglight.length() > 16) {
			clippedTitle = titleHiglight.substring(0, 15);
		}
		String idPrefix = clippedTitle + date;
		return RegisterUtility.generateID(idPrefix, false);

	}

	protected String constructFullTitleFragment(LiteratureTitlePartBean[] fullTitle) {

		StringBuilder fragmentBuilder = new StringBuilder();

		for (int i = 0; i < fullTitle.length; i++) {

			LiteratureTitlePartBean fullTitlePart = fullTitle[i];

			if (fullTitlePart.isHighlighted) {
				fragmentBuilder.append(constructTitleSegment(fullTitlePart.titlePart, "Kennabschnitt"));
			} else {
				fragmentBuilder.append(constructTitleSegment(fullTitlePart.titlePart, "Hauptabschnitt"));
			}
		}

		return fragmentBuilder.toString();

	}

	protected String constructShortTitleFragment(LiteratureTitlePartBean[] shortTitle) {

		StringBuilder fragmentBuilder = new StringBuilder();

		for (int i = 0; i < shortTitle.length; i++) {

			LiteratureTitlePartBean shortTitlePart = shortTitle[i];

			if (shortTitlePart.isHighlighted) {
				fragmentBuilder.append(constructTitleSegment(shortTitlePart.titlePart, "Kennabschnitt"));

			} else {
				fragmentBuilder.append(constructTitleSegment(shortTitlePart.titlePart, "Titelabschnitt"));
			}
		}

		return fragmentBuilder.toString();
	}

	private String constructTitleSegment(String segmentText, String segmentType) {

		StringBuilder segmentBuilder = new StringBuilder();

		segmentBuilder.append("<seg " + XMLUtility.TEIXMLNS + " type='" + segmentType + "'>");
		String escapedSegmentText = XMLUtility.escapeXML(segmentText);
		segmentBuilder.append(escapedSegmentText + "</seg>");

		return segmentBuilder.toString();
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Position darf kein Literaturtitel eingefügt werden.\n "
				+ "Bitte stellen Sie sicher, dass dich der Cursor innerhalb der Literaturliste befindet "
				+ "und nicht innerhalb eines anderen Literaturtitels.";
	}
}
