package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.beans.NewSourceCollectionBean;
import de.historischekommission_muenchen.oxydition.register.beans.SourceCollectionBean;
import de.historischekommission_muenchen.oxydition.register.dialogs.RegisterNewSourceStockDialog;
import de.historischekommission_muenchen.oxydition.register.utilities.RegisterUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Operation mit der die Benutzer*innen einen neuen Bestand-Eintrag in ein
 * Archiv in der RDB für ungedruckte Quellen einfügen können.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterNewSourceStock extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Dialog zum Einfügen eines neuen Eintrags für einen Bestand ungedruckter Quellen";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		boolean newStocksListNeeded = false;

		AuthorElement surroundNode;
		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			AuthorElement archiveNode = validateCursorPosition(surroundNode);

			// archiveNode != null => gültige Position gefunden
			if (archiveNode == null) {
				displayInvalidCaretPositionMessage();
				return;
			}

			newStocksListNeeded = !RegisterUtility.elementContainsListOfType(archiveNode, "Bestände");

			String archiveID = archiveNode.getAttribute("xml:id").getRawValue();
			AuthorElement[] archiveNames = archiveNode.getElementsByLocalName("name");

			SourceCollectionBean archiveBean = new SourceCollectionBean(archiveID, archiveNames);
			RegisterNewSourceStockDialog dlg = new RegisterNewSourceStockDialog(oxygenFrame, archiveBean);

			NewSourceCollectionBean input = dlg.displayDialog();

			// Wenn Archiv noch kein Beständeliste hat, wird sie nun neu
			// angelegt
			if (input.isFilled) {
				if (newStocksListNeeded) {
					insertStockListIntoArchive(documentController, caretPosition);
					caretPosition = editorAccess.getCaretOffset();
				}
				insertNewSourceStock(input, archiveBean.id);
			}

		} catch (

		BadLocationException e) {
			displayBadLocationMessage(e);
		}
	}

	private void insertNewSourceStock(NewSourceCollectionBean input, String archiveId) throws AuthorOperationException {
		String newSourceStockFragment = constructNewSourceStockFragment(input, archiveId);
		documentController.insertXMLFragment(newSourceStockFragment, caretPosition);
	}

	private String constructNewSourceStockFragment(NewSourceCollectionBean input, String archiveId) {

		// ID erzeugen aus ArchivID (ohne Zufallszahl) und Norm-Namen des neuen
		// Eintrags
		String id = archiveId.split("_")[0] + RegisterUtility.generateID(input.normName, false, 20);

		String normName = XMLUtility.escapeXML(input.normName);
		String fullName = XMLUtility.escapeXML(input.fullName);

		String fragment = "<item xmlns='http://www.tei-c.org/ns/1.0' xml:id='" + id + "'>";
		fragment += "<name xmlns='http://www.tei-c.org/ns/1.0' type='Kennname'>" + normName + "</name>";

		// Vollname optional
		if (!fullName.isEmpty()) {
			fragment += "<name xmlns='http://www.tei-c.org/ns/1.0' type='Vollname'>" + fullName + "</name>";
		}

		fragment += "</item>";

		return fragment;
	}

	private void insertStockListIntoArchive(AuthorDocumentController documentController, int caretPosition)
			throws AuthorOperationException {

		String stockListFragment = "<list xmlns='http://www.tei-c.org/ns/1.0' type='Bestände'></list>";

		documentController.insertXMLFragment(stockListFragment, caretPosition);
	}

	// Cursor ist an gültiger Position, wenn er sich innerhalb einer Liste von
	// Beständen (innerhalb eines Archivs) befindet oder innerhalb eines Archivs
	// das noch keine Beständeliste besitzt;
	// Gibt das umgebende Archiv zurück;
	private AuthorElement validateCursorPosition(AuthorElement surroundNode) {

		AuthorElement archiveNode = null;

		if (XMLUtility.elementIsListOfType(surroundNode, "Bestände")) {
			archiveNode = (AuthorElement) surroundNode.getParent();

		} else if (XMLUtility.elementDisplayNameEquals(surroundNode, "item")) {

			AuthorElement surroundNodeParent = (AuthorElement) surroundNode.getParent();

			if (XMLUtility.elementIsListOfType(surroundNodeParent, "Archive")) {

				if (!RegisterUtility.elementContainsListOfType(surroundNode, "Bestände")) {
					archiveNode = surroundNode;
				}
			}
		}
		return archiveNode;

	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

	@Override
	protected Object getInvalidCaretPositionMessage() {
		return "An dieser Stelle kann kein neuer Bestanseintrag eingefügt werden.\n"
				+ "Fügen Sie den Eintrag bitte innerhalb einer bestehenden Bestandliste oder einem Archiveintrag, "
				+ "der noch keine Bestandliste besitzt ein.\n";
	}

}
