package de.historischekommission_muenchen.oxydition.register.utilities;

import java.util.Random;

import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;

/**
 * Hilfsklasse, die statische Methoden zur Verfügung stellt, die bei
 * Registeroperationen zum Einsatz kommen
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterUtility {

	/**
	 * Erzeugt eine sprechende Id für einen neuen Registereintrag, indem es ein
	 * beschreibendes Attribut umwandelt. Dafür wird das Attribut auf
	 * Kleinschreibung umgestellt, alle Sonderzeichen erstetzt, führende Ziffern und
	 * alle anderen zeichen, die nicht zum regulären Alphabet gehören
	 * herausgefilter.Zum Schluss wird noch eine 5stellige Zufallszahl angehängt, um
	 * die Chance auf doppelt vergebene IDs zu minimieren
	 * 
	 * @param inputString - der Attributwert der zu einer ID verarbeitet werden soll
	 *                    preserveLeadingDigits - Singalvariable die anweist ob
	 *                    führende Zahlen beibehalten werden sollen (nur in
	 *                    ausnahmefällen benutzen, da gefahr besteht eine ungültige
	 *                    ID zu erzeugen)
	 * @return der erzeugte ID-Wert
	 */
	public static String generateID(String inputString, boolean preserveLeadingDigits) {

		String id = inputString;

		id = id.toLowerCase();

		id = id.replaceAll("ä", "ae");
		id = id.replaceAll("ö", "oe");
		id = id.replaceAll("ü", "ue");
		id = id.replaceAll("ß", "ss");
		id = id.replaceAll("[^a-z0-9]", "");

		if (!preserveLeadingDigits) {
			id = id.replaceAll("^[\\d]+(?=[a-z])", "");
		}
		int randomIdNumber = new Random().nextInt(90000) + 10000;

		id += "_" + randomIdNumber;

		return id;

	}

	/**
	 * Alternative Funktion zur ID-Erzeugung. Ermöglicht es den Wert der für die
	 * Erzeugung der ID benutzt wird abzuschneiden, wenn er eine gewisse Länge
	 * überschreitet, um übermäßig Lange ID-Werte zu vermeiden
	 * 
	 * @param inputString - der Eingabewert der zu einer ID verarbeitet werden soll
	 * @param maxLength   - die Länge auf die der Eingabetext zugeschnitten werden
	 *                    soll, wenn er diese überschreitet
	 * @return der erzeugte ID-Wert
	 */
	public static String generateID(String inputString, boolean preserveLeadingDigits, int maxLength) {

		if (inputString.length() > maxLength) {
			return generateID(inputString.substring(0, maxLength - 1), preserveLeadingDigits);
		} else {
			return generateID(inputString, preserveLeadingDigits);
		}

	}

	/**
	 * 
	 * Überprüft ob sich ein bestimmter Knoten ein <item>-Knoten ist, der sich
	 * innerhalb eines <list>-Elements mit @type="Sachschlagwörter" befindet oder
	 * sich innerhalb eines solchen <item>-Knoten befindet
	 * 
	 * @param surroundNode - der Knoten dessen Position überprüft werden soll
	 * @return der <item>-Knoten in dem sich der überprüfte Knoten befindet bzw. der
	 *         der überprüfte Knoten ist
	 */
	public static AuthorElement nodeWithinKeyWordItemNode(AuthorElement surroundNode) {

		AuthorElement itemNode = null;
		String surroundNodeDisplayName = "";
		if (surroundNode.getDisplayName() != null) {
			surroundNodeDisplayName = surroundNode.getDisplayName();
		}

		// Wenn sich der Textcursor direkt innerhalb eines
		// <item>-Elements befindet muss der umgebende Knoten eine
		// <list>
		// mit type=Sachschlagwörter sein
		if (surroundNodeDisplayName.equalsIgnoreCase("item")) {

			AuthorElement parentNode = (AuthorElement) surroundNode.getParent();

			if (parentNode.getDisplayName() != null && parentNode.getDisplayName() == "list"
					&& parentNode.getAttribute("type") != null
					&& parentNode.getAttribute("type").getValue().equalsIgnoreCase("Sachschlagwörter")) {
				itemNode = surroundNode;

			}

		}

		// Wenn sich der Textcursor innerhalb eines <ref>-, <note>- oder
		// <term>-Elements befindet muss der umgebende Knoten ein <item>
		// sein und dessen umgebender Knoten wiederrum eine <list> mit
		// @type="Sachschlagwörter"
		else if (surroundNodeDisplayName.equalsIgnoreCase("ref") || surroundNodeDisplayName.equalsIgnoreCase("term")
				|| surroundNodeDisplayName.equalsIgnoreCase("note")) {

			AuthorElement parentNode = (AuthorElement) surroundNode.getParent();

			if (parentNode != null && parentNode.getDisplayName() != null
					&& parentNode.getDisplayName().equalsIgnoreCase("item")) {

				AuthorElement grandParentNode = (AuthorElement) parentNode.getParent();

				if (grandParentNode.getDisplayName() != null && grandParentNode.getDisplayName() == "list"
						&& grandParentNode.getAttribute("type") != null
						&& grandParentNode.getAttribute("type").getValue().equalsIgnoreCase("Sachschlagwörter")) {
					itemNode = parentNode;

				}

			}
		}

		return itemNode;
	}

	/**
	 * Überprüft ob der übergebene Knoten einen bestimmten Kindknoten hat, der
	 * wiederrum ein @type-Attribut mit einem bestimmten Wert hat
	 * 
	 * @param checkNode - Der zu überprüfende Knoten
	 * @param checkName - Der erwartete typ des gesuchten Kindknotens
	 * @param checkAttr - Der erwartete Wert con @type im gesuchten Kindknoten
	 */
	public static boolean hasCertainChildrenWithType(AuthorElement checkNode, String checkName, String checkAttr) {

		boolean has = false;

		AuthorNode[] childrenlist = checkNode.getElementsByLocalName(checkName);

		for (int i = 0; i < childrenlist.length; i++) {
			AuthorElement checkChild = (AuthorElement) childrenlist[i];

			AttrValue checkChildType = checkChild.getAttribute("type");

			if (checkChild != null && checkChildType.getRawValue().equalsIgnoreCase(checkAttr)) {
				has = true;
			}
		}

		return has;

	}

	/**
	 * Prüft ob der übergebene Elementknoten über einen Listenknoten verfügt
	 * dessen @type dem übergebenen type entpricht
	 * 
	 * @param node - der zu überprüfende XML-Knoten
	 * @param type - Der gesuchte Wert des @type-Attributs des überprüften Knoten
	 * 
	 */
	public static boolean elementContainsListOfType(AuthorElement node, String type) {

		AuthorNode[] childLists = node.getElementsByLocalName("list");

		for (int i = 0; i < childLists.length; i++) {

			AuthorElement checkChild = (AuthorElement) childLists[i];

			if (XMLUtility.elementTypeEquals(checkChild, type)) {
				return true;
			}
		}

		return false;
	}
}
